//**************************************************************************
//*								       	                                       *
//*	Copyright JK microsystems 2003	-  All Rights Reserved	  	         *
//*									                                             *
//*	This software is NOT shareware, freeware, or public domain.	         *
//*	It is the property of JK microsystems.				                     *
//*									                                             *
//*	Customers of JK microsystems may modify the source code		         *
//*	and/or distribute the binary image of this software without 	      *
//*	additional costs.  All other use is expressly prohibited             *
//*									                                             *
//**************************************************************************

// CNUT.C is a utility that can be used to communicate with a PowerNut.
//
// This program was written for use on a Flashlite 186 with the following
// connections to a PowerNut:
//
//    Flashlite 186 J9:1  to PowerNut GND     (J3:1)
//    Flashlite 186 J9:5  to PowerNut CLK     (J3:4)
//    Flashlite 186 J9:10 to PowerNut DATA_IO (J3:3)
//
// Mostly, this code exists as sample code for a system developer to start
// with when writing code for a target system other than the Flashlite 186

// Revision list:
//
// The the defined symbol VERSION will be used to print version in PrintUsage()
// The printf that prints VERSION has a %1.2f format specifier.

#define VERSION 1.10

// Revision		Date		Purpose
//    1.00		3/3/3		Initial Release
//    1.10		4/17/03	Fixed a bug in the way the DATA signal was setup.

// INCLUDES GO HERE

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <dos.h>
#include <math.h>

// DEFINES GO HERE

#define PIO0_DataModeReg		0xFF70
#define PIO0_DataDirectionReg	0xFF72
#define PIO0_DataReg				0xFF74

#define PIO1_DataModeReg		0xFF76
#define PIO1_DataDirectionReg	0xFF78
#define PIO1_DataReg          0xFF7A

// FUNCTION PROTOTYPES GO HERE

void CalibrateHalfBitTimeConst( void );
void DelayHalfBit( void );
void WaitOneSecond( void );
int  SendOutPacket( long );
int  TransmitPacket( int , int );
int  chartohex ( char );
int  atox ( char [], int * );
int  OperandValid  ( char [] );
void PrintUsage    ( void );
void initialize_io ( void );
void clear_clk     ( void );
void clear_data    ( void );
void set_clk       ( void );
void set_data      ( void );
int  read_data     ( void );
void make_data_an_input( void );
void make_data_an_output( void );

// GLOBAL DECLARATIONS GO HERE

unsigned int HalfBitTimeConst;

// FUNCTION BODYS GO HERE

int main (int argc, char * argv[])
{
 // The following three declarations must maintain their relationships
 // The enum type is used to index the cmd_string array and the
 // PowerNutOpcode array.
 //
 // For example, cmd_string[LPTEST] corresponds to the ASCII text that
 // is recognized on the command line as PowerNut LPTEST command, while
 // PowerNutOpcode[LPTEST] is the actual opcode that must be sent to the
 // PowerNut to initiate a Low Power Test.

 enum command_type {NOP, SLEEP, CONFIG, LPTEST} command;
 char * cmd_string[] = {"nop", "sleep", "config", "lptest"};
 int PowerNutOpcode[] = {0, 1, 2, 5};

 int ValidCommand;   // just a flag that will be used to track if
							// the ASCII text on the command line equates to
							// an element of cmd_string[]

 int Operand;        // the numeric value of the ASCII comand line text
							// representing the hexadecimal operand.

 initialize_io();    // Configure Flashlite 186 ports for PowerNut interface

 CalibrateHalfBitTimeConst();  	// Figure out what the global
											// HalfBitTimeConst should be and set it up

 // check to see if we have the right number of arguments for a
 // NOP if yes, check for it and if it is there, print the
 // NOP acknowledgement and return without error (return 0)
 if (argc == 2)
	 if (strcmp(argv[1], cmd_string[NOP]) == 0)
		 {printf("\nNothing Happens.\n"); return 0; }

 // check to see if we have the right number of arguments
 // if not, print a "usage" message and return 1
 if (argc != 3) { PrintUsage(); return 1 ; }

 ValidCommand = 0; // do not know that we have a valid command yet.
 if (strcmp( argv[1], cmd_string[SLEEP] ) == 0)
  {command = SLEEP; ValidCommand = 1;}
 if (strcmp( argv[1], cmd_string[CONFIG] ) == 0)
  {command = CONFIG; ValidCommand = 1;}
 if (strcmp( argv[1], cmd_string[LPTEST] ) == 0)
  {command = LPTEST; ValidCommand = 1;}
 if (strcmp( argv[1], cmd_string[NOP] ) == 0)
  {command = NOP; ValidCommand = 1;}
 if (!ValidCommand) { PrintUsage(); return 1 ; }

 if (!atox( argv[2], &Operand ) ) {PrintUsage(); return 1; };

 switch( command ) {
	case SLEEP:
	case CONFIG:
	case LPTEST:
			if ( !TransmitPacket(PowerNutOpcode[command], Operand) ) return 1;
			return 0;
	case NOP:
			printf("\nOperand is:%u",Operand);
			printf("\nNothing Happens.\n");
			return 0;
	default:
			PrintUsage();
			return 1 ;
 }
}

// This function computes a value for the global unsigned int HalfBitTimeConst.
// The value will be used in DelayHalfBit() to burn a little time.  We want
// the value placed in HalfBitTimeConst to be a "count" that is about 1/2 of
// the counts counted in one DOS timer tick.
//
// The DOS timer tick fires 18.2222 times per second which is about 55 ms.
// The idea here is to use the DOS timer tick to generate a time constant
// that can be used in a delay loop in DelayHalfBit() to wait about 25 to 30
// milliseconds.  Ideally we would like our full bit-time (for PowerNut bits)
// to be around 50 ms (see PowerNut Users Guide - section Guidelines for
// Reliability).  Timing in C is not terribly accurate.  This could be done
// in asm better, but the point of this program is to illustrate how it can
// be done in C.
//
//  The idea of using the DOS tick timer to generate the time constant is
//  incorporated here so that someone porting this code to another DOS
//  based controller will not have to worry too much about figuring out how
//  fast the new controller's crystal is running and how that might effect
//  the HalfBitTimeConst selection.  The DOS tick timer provides a ready time
//  reference on all DOS devices.
//
//  This function could be eliminated by simply hard coding a magic number
//  into HalfBitTimeConst.  This might be a useful technique for porting this
//  C-code to a system with out DOS resources (linux for example) or to a DOS
//  system in which the tick timer was disabled, or is being used for a
//  purpose other than generating ticks every 18.222 ms.
//
void CalibrateHalfBitTimeConst( void )
{

extern unsigned int HalfBitTimeConst;

volatile int far * TickCount = MK_FP(0x0040, 0x006c );
int CurrentTick ;

	//  First we let the tick timer roll over so we can get more-or-less
	//  synced to the transition.
	CurrentTick = *TickCount;
	// the following line will cause the compiler to generate a "no effect"
	// warning.  This is normal.  The compiler doesn't see the body of the
	// "while" loop doing anything so it generates a warning.  Of course
	// we know the while loop is really a time delay and is waiting for
	// the Tick Timer to click over.
	while ( *TickCount == CurrentTick); // If Tick Timer is Disabled,
													// this will hang
	HalfBitTimeConst = 0;
	CurrentTick = *TickCount;
	while (*TickCount == CurrentTick)  // If Tick Timer is Disabled,
		HalfBitTimeConst++;             // this will hang

	HalfBitTimeConst = HalfBitTimeConst / 2;  // yes, yes ">> 1" means the
															// same thing...
}

void DelayHalfBit( void )
{
  extern unsigned int HalfBitTimeConst;
  unsigned int delay_counter;
  unsigned int BusyWork4StupidCompiler=0;

  for (delay_counter=0; delay_counter<HalfBitTimeConst; delay_counter++)
	if (BusyWork4StupidCompiler++) BusyWork4StupidCompiler=0;
	//  The "if" statement above is just busy work to while away a few
	//  microseconds.  This is required so time delay works out okay more or
	//  less to be equal to one half a DOS tick.
}

// This function will clock the "long Packet" var out the bit-banged
// serial interface.  The parameter "Packet" will be clocked out LSB
// first.  The required Start-Bit must be the LSB of "Packet."
//
// Upon completion of the transmission of Packet, this function will
// turn around the DATA_IO line and look for an ACK.
// If and ACK is received, this function will print a "suceeded" message
// and terminate.  If a NACK is recieved this function will backoff and
// retry after one second.  If a second NACK is received, then this
// function will print a "failed" message and return with error code set
// (return 0).  If the second attempt resulted in an ACK, this function
// will print a "suceeded" message and indicate it required two trys.
//
// ERROR or failure causes a return value of 0
// SUCCESS causes a return value of 1
//
int SendOutPacket( long Packet )
{
 int counter;
 int AckNack;

 for(counter=0; counter<21; counter++) {
	 if (Packet & 0x00000001)
		{
		 set_data();
		 DelayHalfBit();
		 set_clk();
		}
	 else
		{
		 clear_data();
		 DelayHalfBit();
		 set_clk();
		}
	 DelayHalfBit();
	 clear_clk();
	 Packet = Packet >> 1;
 }
clear_data();
make_data_an_input();
DelayHalfBit();
set_clk();
DelayHalfBit();
AckNack = read_data();
clear_clk();
DelayHalfBit();
make_data_an_output();
set_data();

if (AckNack) return 1;
else return 0;

}
// This function looks at the DOS "tick timer" and waits approximately
// a second before terminating.  This function is called when the
// PowerNut NACKs a packet and the Flashlite 186 wants to wait a second
// before retrying the packet.
//
void WaitOneSecond( void )
{
volatile int far * TickCount = MK_FP(0x0040, 0x006c );
int loop_counter ;
int CurrentTick ;

CurrentTick = *TickCount;

for (loop_counter=0; loop_counter<19; loop_counter++)
	{
	 CurrentTick = *TickCount;
	// the following line will cause the compiler to generate a "no effect"
	// warning.  This is normal.  The compiler doesn't see the body of the
	// "while" loop doing anything so it generates a warning.  Of course
	// we know the while loop is really a time delay and is waiting for
	// the Tick Timer to click over.
	 while (*TickCount == CurrentTick);  // If Tick Timer is Disabled,
													 // this will hang
	}
}

// This function will transmit the command and Operand to the PowerNut.
// If this function receives an ACK from the PowerNut, then the return
// value will be 1.
//
// If the PowerNut returns a NACK, then the return value will be 0
//
// ERROR or failure causes a return value of 0
// SUCCESS causes a return value of 1
//
int TransmitPacket( int Command, int Operand )
{
int TempOperand;
int TempCommand;
int Parity;  // we only care about the LSB of this int.
int i;
long Packet;

TempOperand = Operand;
TempCommand = Command;
Parity = 0;

for(i=0; i<16; i++) {
	 Parity = Parity ^ TempOperand;
	 TempOperand = TempOperand >> 1;
	}

for(i=0; i<3; i++) {
	 Parity = Parity ^ TempCommand;
	 TempCommand = TempCommand >> 1;
	}

Parity = Parity & 0x0001;  // we only care about the LSB of this int.

// Now that we have the Command, Operand and Parity, we need to
// form a PowerNut Packet.

Packet = 0;
Packet = Operand;      		// Put Operand in the Packet
Packet = Packet << 3;      // Scoot it over to make way for the 3-bit Command
Packet = Packet | Command; // OR in the Command bits (3 of them)
Packet = Packet << 1;      // Scoot it over to make way for Parity
Packet = Packet | Parity;  // OR in the Parity bit
Packet = Packet << 1;      // Scoot it over to make way for the Start-Bit
									// Since Start-Bit is ZERO, don't have to do more

if ( SendOutPacket(Packet) )
	{
	 printf("Received ACK, communication Successful\n");
	 return 1;  // tell caller packet succeeded
	}
else
	{
	  // got a NACK
	  WaitOneSecond();
	  if ( SendOutPacket(Packet) )
		  {
			 printf("Received ACK after one retry - communication successful.\n");
			 return 1;  // tell caller packet succeeded
		  }
	  else
		  {
			 printf("Received NACK, retried once, received second NACK.\n");
			 printf("Communication FAILED.\n");
			 return 0;  // tell caller packet failed
		  }
	}
}

// This function converts an alpha string in a hexadecimal format
// to an int.  The format restrictions are
// Leading zeros are accepted but ignored
// No "0x" prefix is allowed
// No "h" suffix is allowed
// The string length must be between 1 and 4 digits
// the digits must be elements of {A..F, a..f, 0..9}
//
// if the string has an invalid format this function returns 0
// else, this function returns 1
//
// ERROR or failure causes a return value of 0
// SUCCESS causes a return value of 1
//
int  atox ( char TestString[], int * NumericResult )
{
 int OperandLength;
 int count;

 if (!OperandValid( TestString ) ) return 0;

 // now that we know we have an operand with a valid format
 // we'll convert the alpha to hex

	OperandLength = strlen(TestString);
 * NumericResult = 0;
	count = 0;

 while (OperandLength) {
  * NumericResult = * NumericResult +
						  ((int) pow(16.0, count++) *
							chartohex(TestString[--OperandLength]));
  }

  return 1;
}

// This function converts an ASCII char representing a hexadecimal
// digit into a numeric representation that is passed back to the
// caller as the return value.
//
// If CharInQuestion is not a valid hex digit, the return value
// will be zero.  The ASCII char passed into this function should
// be qualified as a legitimate hexidecimal char before being
// passed to this function.
//
int chartohex ( char CharInQuestion )
{
 switch (CharInQuestion) {
 case '0': return 0;
 case '1': return 1;
 case '2': return 2;
 case '3': return 3;
 case '4': return 4;
 case '5': return 5;
 case '6': return 6;
 case '7': return 7;
 case '8': return 8;
 case '9': return 9;
 case 'A': return 10;
 case 'a': return 10;
 case 'B': return 11;
 case 'b': return 11;
 case 'C': return 12;
 case 'c': return 12;
 case 'D': return 13;
 case 'd': return 13;
 case 'E': return 14;
 case 'e': return 14;
 case 'F': return 15;
 case 'f': return 15;
 default: return 0;
 }
}

// This function checks to see if the operand has 1-4 digits
// and if all digits are an element of {A..F, a..f, 0..9).
// If all conditons are met, this function returns 1, else 0
//
// ERROR or failure causes a return value of 0
// SUCCESS causes a return value of 1
//
int OperandValid( char TestString[] )
{
 int OperandLength;

 OperandLength = strlen(TestString);

 // do we have a valid length?
 if ( (OperandLength < 1) || (OperandLength > 4) ) return 0;

 // do we have valid digits?
 while (OperandLength)
  if (!isxdigit(TestString[--OperandLength])) return 0;

return 1;
}

// This function prints out a usage message for users
void PrintUsage( void )
{
printf("PowerNut Communications Utility for Flashlite 186\n");
printf("Version %1.2f - see www.jkmicro.com for updates\n", VERSION);
printf("\n");
printf("Usage: cnut <command> <param>\n");
printf("  commands - sleep, config, lptest\n");
printf("  param - hex value between 0000 and FFFF\n");
printf("        - do NOT prefix with 0x or suffix with 'h'\n");
printf("        - leading zeros are accepted but not required\n");
printf("  EXAMPLE \"cnut sleep, 000A\" - means sleep for 10 seconds\n");
printf("\n");
printf("  Required electrical connections:\n");
printf("    Flashlite 186 J9:1  to PowerNut GND     (J3:1)\n");
printf("    Flashlite 186 J9:5  to PowerNut CLK     (J3:4)\n");
printf("    Flashlite 186 J9:10 to PowerNut DATA_IO (J3:3)\n");
}



// This function sets up PIO 1 bit 15 and PIO 0 bit 1
// to be outputs that will drive the PowerNut's
// CLK and DATA_IO signals.
void initialize_io (void)
{
 int temp;

 // set up the PIO 1 bit 15 to be PowerNut's CLK
 // On the Flashlite 186, this bit comes out on J9 pin 5
 // This is a "multi function" pin, see RDC or AMD datasheet

	temp = inport( PIO1_DataModeReg );  		// set PIO 1
	temp = temp | 0x8000 ;              		// bit 15
	outport( PIO1_DataModeReg, temp );    		// to mode 1

	temp = inport( PIO1_DataDirectionReg );   // set PIO 1
	temp = temp & 0x7FFF	;							// bit 15
	outport( PIO1_DataDirectionReg, temp );	// to be an output

	temp = inport( PIO1_DataReg );				// set PIO 1
	temp = temp & 0x7FFF ;                    // bit 15
	outport( PIO1_DataReg, temp );            // to be LOW

 // set up the PIO 0 bit 1 to be PowerNut's DATA_IO
 // On the Flashlite 186, this bit comes out on J9 pin 10
 // This is a "multi function" pin, see RDC or AMD datasheet

	temp = inport( PIO0_DataModeReg );  		// set PIO 0
	temp = temp | 0x0002 ;              		// bit 1
	outport( PIO0_DataModeReg, temp );    		// to mode 1

	temp = inport( PIO0_DataDirectionReg );   // set PIO 0
	temp = temp & 0xFFFD	;							// bit 1
	outport( PIO0_DataDirectionReg, temp );	// to be an output

	temp = inport( PIO0_DataReg );				// set PIO 0
	temp = temp & 0xFFFD ;                    // bit 1
	outport( PIO0_DataReg, temp );            // to be LOW

}

// This function drives the PowerNut's CLK line LOW.
void clear_clk ( void )
{
 int temp;
	temp = inport( PIO1_DataReg );				// set PIO 1
	temp = temp & 0x7FFF ;                    // bit 15
	outport( PIO1_DataReg, temp );            // to be LOW
}

// This function drives the PowerNut's CLK line HIGH.
void set_clk ( void )
{
 int temp;
	temp = inport( PIO1_DataReg );				// set PIO 1
	temp = temp | 0x8000 ;                    // bit 15
	outport( PIO1_DataReg, temp );            // to be HIGH
}

// This function drives the PowerNut's DATA_IO line LOW.
void clear_data ( void )
{
 int temp;
	temp = inport( PIO0_DataReg );				// set PIO 0
	temp = temp & 0xFFFD ;                    // bit 1
	outport( PIO0_DataReg, temp );            // to be LOW
}

// This function drives the PowerNut's DATA_IO line HIGH.
void set_data ( void )
{
 int temp;
	temp = inport( PIO0_DataReg );				// set PIO 0
	temp = temp | 0x0002 ;                    // bit 1
	outport( PIO0_DataReg, temp );            // to be LOW
}

// This function turns the PIO pin attached to the PowerNut's
// DATA_IO into an input - used in preparation for reading an
// ACK or NACK from the PowerNut.
void make_data_an_input (void)
{
 int temp;
	temp = inport( PIO0_DataDirectionReg );   // set PIO 0
	temp = temp | 0x0002	;							// bit 1
	outport( PIO0_DataDirectionReg, temp );	// to be an input
}

// This function turns the PIO pin attached to the PowerNut's
// DATA_IO into an output.  This function assumes the mode
// register has already been set up by a call to
// initialize_io().  This function also forces the PIO pin to
// drive the PowerNut's DATA_IO signal low.
void make_data_an_output (void)
{
 int temp;
	temp = inport( PIO0_DataDirectionReg );   // set PIO 0
	temp = temp & 0xFFFD	;							// bit 1
	outport( PIO0_DataDirectionReg, temp );	// to be an output
	clear_data();                             // and force it LOW
}

// This function returns the value on the PowerNut's DATA_IO
// signal.  This function assumes the PIO pin has already been
// setup to be an input with a call to make_data_an_input.
// This function is used to read the ACK or NACK from the
// PowerNut.
int read_data (void)
{
 int temp;
 temp = inport( PIO0_DataReg );
 temp = temp & 0x0002 ;
 temp = temp >> 1;
 return temp;
}
